import 'dart:math';

import '../extensions/extensions.dart';

class Line {
  final double a;
  final double b;
  final double c;

  const Line(this.a, this.b, this.c);

  static Line fromPoints(Vector2 p1, Vector2 p2) {
    final a = p2.y - p1.y;
    final b = p1.x - p2.x;
    final c = p2.y * p1.x - p1.y * p2.x;
    return Line(a, b, c);
  }

  List<Vector2> intersections(Line otherLine) {
    final determinant = a * otherLine.b - otherLine.a * b;
    if (determinant == 0) {
      //The lines are parallel (potentially coincides) and have no intersection
      return [];
    }
    return [
      Vector2(
        (otherLine.b * c - b * otherLine.c) / determinant,
        (a * otherLine.c - otherLine.a * c) / determinant,
      ),
    ];
  }

  double get angle => atan2(-a, b);

  @override
  String toString() {
    final ax = '${a}x';
    final by = b.isNegative ? '${b}y' : '+${b}y';
    return '$ax$by=$c';
  }
}
