import '../extensions/extensions.dart';
import 'line.dart';

class LineSegment {
  final Vector2 from;
  final Vector2 to;

  LineSegment(this.from, this.to);

  factory LineSegment.zero() => LineSegment(Vector2.zero(), Vector2.zero());

  List<Vector2> intersections(LineSegment otherSegment) {
    final result = toLine().intersections(otherSegment.toLine());
    if (result.isNotEmpty) {
      final intersection = result.first;
      if (containsPoint(intersection) && otherSegment.containsPoint(intersection)) {
        return result;
      }
    }

    return [];
  }

  bool crossed(LineSegment otherSegment) {
    return intersections(otherSegment).isNotEmpty;
  }

  bool containsPoint(Vector2 point, {double epsilon = 0.000001}) {
    final delta = to - from;
    final crossProduct = (point.y - from.y) * delta.x - (point.x - from.x) * delta.y;

    // compare versus epsilon for floating point values
    if (crossProduct.abs() > epsilon) {
      return false;
    }

    final dotProduct = (point.x - from.x) * delta.x + (point.y - from.y) * delta.y;
    if (dotProduct < 0) {
      return false;
    }

    final squaredLength = from.distanceToSquared(to);
    if (dotProduct > squaredLength) {
      return false;
    }

    return true;
  }

  bool pointsAt(Line line) {
    final result = toLine().intersections(line);
    if (result.isNotEmpty) {
      final delta = to - from;
      final intersection = result.first;
      final intersectionDelta = intersection - to;

      if (delta.x.sign == intersectionDelta.x.sign && delta.y.sign == intersectionDelta.y.sign) {
        return true;
      }
    }
    return false;
  }

  Line toLine() => Line.fromPoints(from, to);

  @override
  String toString() {
    return '[$from, $to]';
  }
}
